pro civ_subtract, index, data,  $
                  iout, dout, $
                  err   = err, $
                  shift = shift, $
                  debug = debug
;+
; NAME:
;    CIV_SUBTRACT.PRO
;
; PURPOSE:
;    Given an arbitrary set of UV images from TRACE out of the 
;    1550, 1600 and 1700 Images, generate a "Clean" C IV image, 
;    per Handy et al. (1998) Solar Physics, in press.
;
;    Note that unlike Handy et al., this routine also includes 
;    a capability to extract CIV images from any arbitrary pair of
;    UV images.  This has not yet been fully investigated, and the 
;    uncertainty in these measurements is pretty suspect.
;
; CALLING SEQUENCE:
;    civ_subtract, index, data, iout, dout [,err=err][,/debug]
; 
; INPUTS:
;    index, data -- data cube for the pair/triplet of images
;                   (data cube is expected to be correlated, images 
;                   are expected to be the same size, but order of 
;                   images (out of 1550, 1600, 1700) is arbitrary
;
; KEYWORD PARAMETERS:
;    shift       -- If set, CIV_SUBTRACT will coalign the image set 
;                   prior to doing the analysis routine, using 
;                   TR_GET_DISP.PRO.
;    debug       -- Gives small, less-than-useful hints as to what's 
;                   going on.
;
; OUTPUTS:
;    iout, dout  -- index and data element of C IV image.  The units 
;                   of dout are 10E12 photons/s/cm^2/Sr.  iout is currently
;                   a copy of the index structure of the 1550 image, or
;                   the 1600 image if 1550 isn't used.
;
; OPTIONAL OUTPUTS:
;    err         -- The predicted error of the image  (tbd)
;
; TO DO:
;    Which images (and their dates, and other information) needs to be 
;    included in the history of the returned index structure.
;
; MODIFICATION HISTORY:
;    19-Jul-98 - (BNH) - Written
;    24-Jul-98 - (BNH) - Do the image normalization here.  That seems to
;                        be the method of least surprise.  Also fix one
;                        typo.
;     3-Aug-98 - (BNH) - Use 'index' everwhere to fix exposure normalization.
;    30-Sep-98 - (BNH) - Two changes, one BIG, one small:
;                      - By popular demand, return IOUT in units of
;                        10^12 Photons/Pixel/etc.
;                      - Coefficients on everything for the 2-image sets
;                        was just wrong.  Fixed.
;     2-Oct-98 - (BNH) - Added /shift keyword.
;                      - Added a version number.
;                      - Record version number and frames used in 
;                        new index tag index.HIST_CIV.
;
; $Id: civ_subtract.pro,v 1.6 1998/07/24 16:50:09 handy Exp $
;-
civ_version = 1.8
debug = keyword_set(debug)

; Constants
i2c_corr =  1.07
ss_1550 = where(index.wave_len eq '1550', )
ss_1600 = where(index.wave_len eq '1600')
ss_1700 = where(index.wave_len eq '1700')

;; 
;; Determine which set of coefficients to use
;;
case 1 of
    ;; Normal Case:  Three images, 1550, 1600, 1700
    ((ss_1550[0] ge 0) and (ss_1600[0] ge 0) and (ss_1700[0] ge 0)): begin
        if keyword_set(debug) then $
          message, /info, 'Using 1550/1600/1700 image set.'

        if keyword_set(shift) then begin
            offset_1 = tr_get_disp(data[*,*,[ss_1550,ss_1600]], /shift,debug=debug)
            offset_2 = tr_get_disp(data[*,*,[ss_1600,ss_1700]], /shift,debug=debug)
        endif

        i1 = data[*,*,ss_1550]/index[ss_1550].sht_mdur
        i2 = data[*,*,ss_1600]/index[ss_1600].sht_mdur - $
             data[*,*,ss_1700]/index[ss_1700].sht_mdur * i2c_corr
        iout = index[ss_1550]

        alpha =  15.5
        beta  =  -1.09
    end

    ;; 1550 and 1600 Images
    ((ss_1550[0] ge 0) and (ss_1600[0] ge 0) and (ss_1700[0] lt 0)): begin
        if keyword_set(debug) then $
          message, /info, 'Using 1550/1600 image set.'

        if keyword_set(shift) then $
          offset_1 = tr_get_disp(data[*,*,[ss_1550,ss_1600]], /shift, debug=debug)

        i1 = data[*,*,ss_1550]/index[ss_1550].sht_mdur
        i2 = data[*,*,ss_1600]/index[ss_1600].sht_mdur
        iout = index[ss_1550]

        alpha =  13.2
        beta  =  -7.28
    end

    ;; 1550 and 1700 Images
    ((ss_1550[0] ge 0)and (ss_1600[0] lt 0) and (ss_1700[0] ge 0)): begin
        if keyword_set(debug) then $
          message, /info, 'Using 1550/1700 image set.'

        if keyword_set(shift) then $
          offset_1 = tr_get_disp(data[*,*,[ss_1550,ss_1700]], /shift, debug=debug)

        i1 = data[*,*,ss_1550]/index[ss_1550].sht_mdur
        i2 = data[*,*,ss_1700]/index[ss_1700].sht_mdur
        iout = index[ss_1550]

        alpha =  8.55
        beta  = -2.34
    end

    ;; 1600 and 1700 Images
    ((ss_1550[0] lt 0) and (ss_1600[0] ge 0) and (ss_1700[0] ge 0)): begin
        if keyword_set(debug) then $
          message, /info, 'Using 1600/1700 image set.'

        if keyword_set(shift) then $
          offset_1 = tr_get_disp(data[*,*,[ss_1600,ss_1700]], /shift, debug=debug)

        i1 = data[*,*,ss_1600]/index[ss_1600].sht_mdur
        i2 = data[*,*,ss_1700]/index[ss_1700].sht_mdur
        iout = index[ss_1600]

        alpha =  1.33
        beta  = -6.60
    end

    else:  message, 'Insufficient data cube.'

endcase

dout = (alpha * i1) + (beta * i2)

iout.wave_len = 'C IV'
hist_civ = ['CIV_SUBTRACT VERSION = '+string(civ_version,format='(f5.2)')]

if (ss_1550[0] ne -1) then $
    hist_civ = [hist_civ, 'Image_1550 = '+fmt_tim(index[ss_1550])]
if (ss_1600[0] ne -1) then $
    hist_civ = [hist_civ, 'Image_1600 = '+fmt_tim(index[ss_1600])]
if (ss_1700[0] ne -1) then $
    hist_civ = [hist_civ, 'Image_1700 = '+fmt_tim(index[ss_1700])]

iout = add_tag(iout, hist_civ, 'hist_civ')

return
end
